/*
  CjpegRJ - Based on Cjpeg 6b AltUI [supports wild cards via SETARGV.OBJ MS-C++ V6].
          - Supports Long File Names.
          - Use Double Quotes around filenames with Embedded Spaces.
          - Outfile can be a folder name ending in slash.
          - Syntax info is now context sensitive.
          - Minimum abbreviations are indicated with Capitals on the help screen.
          - More info in file error messages.
          - Single keystroke for y/n response.
          - if error occurs, it waits for keyboard before closing window.
          - Huffman Optimization turned ON by DEFAULT.
          - Tested on Windows XP and 98SE.  Runs fast. 
          - Does NOT run in 16-bit Native Dos mode.

 * Alternate cjpeg.c
 *
 * Copyright (C) 1991-1998, Thomas G. Lane.
 * This file is mostly based on the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file contains an alternate user interface for the JPEG compressor.
 * One or more input files are named on the command line, and output file
 * names are created by substituting ".jpg" for the input file's extension.
 */

#include "cdjpeg.h"                  /* Common decls for cjpeg/djpeg applications */
#include "jversion.h"                  /* for version message */

#ifdef USE_CCOMMAND                  /* command-line reader for Macintosh */
#ifdef __MWERKS__
#include <SIOUX.h>              /* Metrowerks needs this */
#include <console.h>                  /* ... and this */
#endif
#ifdef THINK_C
#include <console.h>                  /* Think declares it here */
#endif
#endif

#ifndef PATH_MAX                  /* ANSI maximum-pathname-length constant */
#define PATH_MAX 256
#endif

/* Create the add-on message string table. */

#define JMESSAGE(code,string)         string ,

static const char * const cdjpeg_message_table[] = 
{
#include "cderror.h"
  NULL
};


/*
 * Automatic determination of available memory.
 */

static long default_maxmem;         /* saves value determined at startup, or 0 */

#ifndef FREE_MEM_ESTIMATE         /* may be defined from command line */

#ifdef MSDOS                           /* For MS-DOS (unless flat-memory model) */

#include <dos.h>                  /* for access to intdos() call */

LOCAL(long)
unused_dos_memory (void)
/* Obtain total amount of unallocated DOS memory */
{
  union REGS regs;
  long nparas;

  regs.h.ah = 0x48;                  /* DOS function Allocate Memory Block */
  regs.x.bx = 0xFFFF;                  /* Ask for more memory than DOS can have */
  (void) intdos(&regs, &regs);
  /* DOS will fail and return # of paragraphs actually available in BX. */
  nparas = (unsigned int) regs.x.bx;
  /* Times 16 to convert to bytes. */
  return nparas << 4;
}

// The default memory setting is 95% of the available space. 
#define FREE_MEM_ESTIMATE  ((unused_dos_memory() * 95L) / 100L)

#endif /* MSDOS */

#ifdef ATARI                           /* For Atari ST/STE/TT, Pure C or Turbo C */

#include <ext.h>

/* The default memory setting is 90% of the available space. */
#define FREE_MEM_ESTIMATE  (((long) coreleft() * 90L) / 100L)

#endif /* ATARI */

/* Add memory-estimation procedures for other operating systems here,
 * with appropriate #ifdef's around them.
 */

#endif /* !FREE_MEM_ESTIMATE */


/*
 * This routine determines what format the input file is,
 * and selects the appropriate input-reading module.
 *
 * To determine which family of input formats the file belongs to,
 * we may look only at the first byte of the file, since C does not
 * guarantee that more than one character can be pushed back with ungetc.
 * Looking at additional bytes would require one of these approaches:
 *     1) assume we can fseek() the input file (fails for piped input);
 *     2) assume we can push back more than one character (works in
 *        some C implementations, but unportable);
 *     3) provide our own buffering (breaks input readers that want to use
 *        stdio directly, such as the RLE library);
 * or  4) don't put back the data, and modify the input_init methods to assume
 *        they start reading after the start of file (also breaks RLE library).
 * #1 is attractive for MS-DOS but is untenable on Unix.
 *
 * The most portable solution for file types that can't be identified by their
 * first byte is to make the user tell us what they are.  This is also the
 * only approach for "raw" file types that contain only arbitrary values.
 * We presently apply this method for Targa files.  Most of the time Targa
 * files start with 0x00, so we recognize that case.  Potentially, however,
 * a Targa file could start with any byte value (byte 0 is the length of the
 * seldom-used ID field), so we provide a switch to force Targa input mode.
 */

static boolean is_targa;         /* records user -targa switch */

void PressAnyKey()
{
  char cT;
  fprintf(stderr, "                      --- press any key ---");
  getche(cT);
  fprintf(stderr, "\n");
}


LOCAL(cjpeg_source_ptr)
select_file_type (j_compress_ptr cinfo, FILE * infile)
{
  int c;

  if (is_targa) 
  {
#ifdef TARGA_SUPPORTED
    return jinit_read_targa(cinfo);
#else
    ERREXIT(cinfo, JERR_TGA_NOTCOMP);
#endif
  }

  if ((c = getc(infile)) == EOF)
    ERREXIT(cinfo, JERR_INPUT_EMPTY);

  if (ungetc(c, infile) == EOF)
    ERREXIT(cinfo, JERR_UNGETC_FAILED);

  switch (c) 
  {
#ifdef BMP_SUPPORTED
  case 'B':
    return jinit_read_bmp(cinfo);
#endif
#ifdef GIF_SUPPORTED
  case 'G':
    return jinit_read_gif(cinfo);
#endif
#ifdef PPM_SUPPORTED
  case 'P':
    return jinit_read_ppm(cinfo);
#endif
#ifdef RLE_SUPPORTED
  case 'R':
    return jinit_read_rle(cinfo);
#endif
#ifdef TARGA_SUPPORTED
  case 0x00:
    return jinit_read_targa(cinfo);
#endif
  default:
    ERREXIT(cinfo, JERR_UNKNOWN_FORMAT);
    break;
  }

  return NULL;                           /* suppress compiler warnings */
}


/*
 * Argument-parsing code.
 * The switch parser is designed to be useful with DOS-style command line
 * syntax, ie, intermixed switches and file names, where only the switches
 * to the left of a given file name affect processing of that file.
 */


static const char *lsProgName;         /* program name for error messages */
static char *lsOUT_FileName;         /* for -outfile switch */
static char *lsOUT_NameBegin;
int  iDSN_NameLen = 0;
char *DSN_Spreader = &" ";
char cOUT_Wild_Flag;


LOCAL(void) usage (char cP1)
/* complain about bad command line */
{

  if (cP1 == 0)
  {
      fprintf(stderr, "USAGE: %s [switches] inputfile(s)\n", lsProgName);
      fprintf(stderr, "Input filenames may use wildcards (* and ?)\n");
      fprintf(stderr, "Output filename defaults to input name with .jpg\n");
  }
  if (cP1 == 'q' || cP1 < 2)
  {
      fprintf(stderr, "  -Quality N     Compression quality 0..100   Default=75 Yuck<50 Silly>95\n");
  }
  if (cP1 == 'g' || cP1 < 2)
  {
      fprintf(stderr, "  -GRayscale     Create monochrome JPEG file\n");
  }
#ifdef C_PROGRESSIVE_SUPPORTED
  if (cP1 == 'p' || cP1 < 2)
  {
      fprintf(stderr, "  -Progressive   Create progressive JPEG file\n");
  }
#endif
#ifdef INPUT_SMOOTHING_SUPPORTED
  if (cP1 == 's' || cP1 < 2)
  {
      fprintf(stderr, "  -SMooth N      Smooth dithered input (N=1..100 is strength)\n");
  }
#endif
  if (cP1 == 'o' || cP1 < 2)
  {
      fprintf(stderr, "  -Outfile name  If specfified, must PRECEDE input file name\n");
  }
#ifdef ENTROPY_OPT_SUPPORTED
  if (cP1 == 'o' || cP1 < 2)
  {
      fprintf(stderr, "  -NOoptimize    NO Optimize Huffman table (faster, but bigger file)\n");
  }
#endif
#ifdef TARGA_SUPPORTED
  if (cP1 == 't' || cP1 < 2)
  {
      fprintf(stderr, "  -TARga         Input file is Targa format (usually not needed)\n");
  }
#endif
  if (cP1 < 2)
  {
      fprintf(stderr, "Advanced Switches:\n");
  }
#ifdef DCT_ISLOW_SUPPORTED
  if (cP1 == 'd' || cP1 < 2)
  {
      fprintf(stderr, "  -dct int       Use integer DCT method%s\n",
           (JDCT_DEFAULT == JDCT_ISLOW ? " (default)" : ""));
  }
#endif
#ifdef DCT_IFAST_SUPPORTED
  if (cP1 == 'd' || cP1 < 2)
  {
      fprintf(stderr, "  -dct fast      Use fast integer DCT (less accurate)%s\n",
           (JDCT_DEFAULT == JDCT_IFAST ? " (default)" : ""));
  }
#endif
#ifdef DCT_FLOAT_SUPPORTED
  if (cP1 == 'd' || cP1 < 2)
  {
      fprintf(stderr, "  -dct float     Use floating-point DCT method%s\n",
           (JDCT_DEFAULT == JDCT_FLOAT ? " (default)" : ""));
  }
#endif
  if (cP1 == 'r' || cP1 < 2)
  {
      fprintf(stderr, "  -Restart N     Set restart interval in rows, or in blocks with B\n");
  }
#ifndef FREE_MEM_ESTIMATE
  if (cP1 == 'm' || cP1 < 2)
  {
      fprintf(stderr, "  -MAXmemory N   Maximum memory to use (in kbytes)\n");
  }
#endif
  if (cP1 == 'v' || cP1 < 2)
  {
      fprintf(stderr, "  -DBG           Show debug info\n");
  }
#ifdef C_ARITH_CODING_SUPPORTED
  if (cP1 == 'a' || cP1 < 2)
  {
      fprintf(stderr, "  -Arithmetic    Use arithmetic coding\n");
  }
#endif
  if (cP1 == 'b' || cP1 < 2)
  {
      fprintf(stderr, "  -Baseline      Force baseline quantization tables\n");
  }
  if (cP1 == 'q' || cP1 < 2)
  {
      fprintf(stderr, "  -QTables file  Use quantization tables given in file\n");
  }
  if (cP1 == 'q' || cP1 < 2)
  {
      fprintf(stderr, "  -QSlots N[,...]    Set component quantization tables\n");
  }
  if (cP1 == 's' || cP1 < 2)
  {
      fprintf(stderr, "  -Sample HxV[,...]  Set component sampling factors\n");
  }
#ifdef C_MULTISCAN_FILES_SUPPORTED
  if (cP1 == 's' || cP1 < 2)
  {
      fprintf(stderr, "  -SCans file    Create multi-scan JPEG per script file\n");
  }
#endif
  PressAnyKey();
  exit(EXIT_FAILURE);
}



LOCAL(int)
parse_switches (j_compress_ptr cinfo, int argc, char **argv,
                  int last_file_arg_seen, boolean for_real)
/* Parse optional switches.
 * Returns argv[] index of first file-name argument (== argc if none).
 * Any file names with indexes <= last_file_arg_seen are ignored;
 * they have presumably been processed in a previous iteration.
 * (Pass 0 for last_file_arg_seen on the first or only iteration.)
 * for_real is FALSE on the first (dummy) pass; we may skip any expensive
 * processing.
 */
{
  int argn;
  char * arg;
  int quality;                           /* -quality parameter */
  int q_scale_factor;                  /* scaling percentage for -qtables */
  boolean force_baseline;
  boolean simple_progressive;
  char * qtablefile = NULL;         /* saves -qtables filename if any */
  char * qslotsarg = NULL;         /* saves -qslots parm if any */
  char * samplearg = NULL;         /* saves -sample parm if any */
  char * scansarg = NULL;         /* saves -scans parm if any */

  /* Set up default JPEG parameters. */
  /* Note that default -quality level need not, and does not,
   * match the default scaling for an explicit -qtables argument.
   */

  quality = 75;                           /* default -quality value */
  q_scale_factor = 100;                  /* default to no scaling for -qtables */
  force_baseline = FALSE;         /* by default, allow 16-bit quantizers */
  simple_progressive = FALSE;
  is_targa = FALSE;
  lsOUT_FileName = NULL;
  cinfo->err->trace_level = 0;
  if (default_maxmem > 0)         /* override library's default value */
    cinfo->mem->max_memory_to_use = default_maxmem;

  // Default Entropy Optimization to TRUE
  cinfo->optimize_coding = TRUE;

  /* Scan command line options, adjust parameters */

  for (argn = 1; argn < argc; argn++) 
  {
    arg = argv[argn];
    if (*arg != '-') 
    {
      // Not a switch, must be a file name argument 
      if (argn <= last_file_arg_seen) 
      {
          if (!cOUT_Wild_Flag)
              lsOUT_FileName = NULL;      // -outfile applies to just one input file 
         continue;                  // ignore this name if previously processed 
      }
      break;                           // else done parsing switches 
    }
    arg++;                           // advance past switch marker character 

    if (keymatch(arg, "arithmetic", 1)) 
    {
      /* Use arithmetic coding. */
#ifdef C_ARITH_CODING_SUPPORTED
      cinfo->arith_code = TRUE;
#else
      fprintf(stderr, "%s: sorry, arithmetic coding not supported\n",
               lsProgName);
      PressAnyKey();
      exit(EXIT_FAILURE);
#endif

    } 
    else 
    if (keymatch(arg, "baseline", 1)) 
    {
      /* Force baseline-compatible output (8-bit quantizer values). */
      force_baseline = TRUE;

    } 
    else 
    if (keymatch(arg, "dct", 2)) 
    {
      /* Select DCT algorithm. */
      if (++argn >= argc)         /* advance to next argument */
         usage('d');
      if (keymatch(argv[argn], "int", 1)) 
      {
         cinfo->dct_method = JDCT_ISLOW;
      } 
      else 
      if (keymatch(argv[argn], "fast", 2)) 
      {
         cinfo->dct_method = JDCT_IFAST;
      } 
      else
      if (keymatch(argv[argn], "float", 2)) 
      {
         cinfo->dct_method = JDCT_FLOAT;
      } else
         usage('d');

    } 
    else 
    if (keymatch(arg, "debug", 5) || keymatch(arg, "dbg", 3)
     || keymatch(arg, "verbose", 3)) 
    {
      /* Enable debug printouts. */
      /* On first -d, print version identification */
      static boolean printed_version = FALSE;

      if (! printed_version) 
      {
         
         char szTemp[32];
                  // Get compilation date
         strcpy( szTemp, "YYYY-MMM-DD");
         memcpy( &szTemp[5], &__DATE__,    3 );
         memcpy( &szTemp[9], &__DATE__[4], 2 );
         memcpy( &szTemp,    &__DATE__[7], 4 );
         
         fprintf(stderr, "Independent JPEG Group CJPEG, ver %s %s\n\t%s\n",
                  JVERSION, szTemp, JCOPYRIGHT);

         printed_version = TRUE;
      }
      cinfo->err->trace_level++;

    } 
    else 
    if (keymatch(arg, "grayscale", 2) 
     || keymatch(arg, "greyscale", 2)) 
    {
      /* Force a monochrome JPEG file to be generated. */
      jpeg_set_colorspace(cinfo, JCS_GRAYSCALE);

    } 
    else 
    if (keymatch(arg, "maxmemory", 3)) 
    {
      /* Maximum memory in Kb (or Mb with 'm'). */
      long lval;
      char ch = 'x';

      if (++argn >= argc)         /* advance to next argument */
         usage('m');
      if (sscanf(argv[argn], "%ld%c", &lval, &ch) < 1)
         usage('m');
      if (ch == 'm' || ch == 'M')
         lval *= 1000L;
      cinfo->mem->max_memory_to_use = lval * 1000L;

    } 
    else 
    if (keymatch(arg, "optimize", 3) 
     || keymatch(arg, "optimise", 3)) 
    {
      /* Enable entropy parm optimization. */
#ifdef ENTROPY_OPT_SUPPORTED
      cinfo->optimize_coding = TRUE;
#else
      fprintf(stderr, "%s: sorry, entropy optimization was not compiled\n",
               lsProgName);
      PressAnyKey();
      exit(EXIT_FAILURE);
#endif

    } 
    else 
    if (keymatch(arg, "nooptimize", 3) 
     || keymatch(arg, "nooptimise", 3)) 
    {
      /* DISABLE entropy parm optimization. */
      cinfo->optimize_coding = FALSE;
    } 
    else 
    if (keymatch(arg, "outfile", 1)) 
    {
      /* Set output file name. */
      if (++argn >= argc)         /* advance to next argument */
         usage('o');
      lsOUT_FileName = argv[argn];         /* save it away for later use */

    } 
    else 
    if (keymatch(arg, "progressive", 1)) 
    {
      /* Select simple progressive mode. */
#ifdef C_PROGRESSIVE_SUPPORTED
      simple_progressive = TRUE;
      /* We must postpone execution until num_components is known. */
#else
      fprintf(stderr, "%s: sorry, progressive output was not compiled\n",
               lsProgName);
      PressAnyKey();
      exit(EXIT_FAILURE);
#endif

    } 
    else 
    if (keymatch(arg, "quality", 1)) 
    {
      /* Quality factor (quantization table scaling factor). */
      if (++argn >= argc)         /* advance to next argument */
         usage('q');
      if (sscanf(argv[argn], "%d", &quality) != 1)
         usage('q');
      /* Change scale factor in case -qtables is present. */
      q_scale_factor = jpeg_quality_scaling(quality);

    } 
    else 
    if (keymatch(arg, "qslots", 2)) 
    {
      /* Quantization table slot numbers. */
      if (++argn >= argc)         /* advance to next argument */
         usage('q');
      qslotsarg = argv[argn];
      /* Must delay setting qslots until after we have processed any
       * colorspace-determining switches, since jpeg_set_colorspace sets
       * default quant table numbers.
       */

    } 
    else 
    if (keymatch(arg, "qtables", 2)) 
    {
      /* Quantization tables fetched from file. */
      if (++argn >= argc)         /* advance to next argument */
         usage('q');
      qtablefile = argv[argn];
      /* We postpone actually reading the file in case -quality comes later. */

    } 
    else 
    if (keymatch(arg, "restart", 1)) 
    {
      /* Restart interval in MCU rows (or in MCUs with 'b'). */
      long lval;
      char ch = 'x';

      if (++argn >= argc)         /* advance to next argument */
         usage('r');
      if (sscanf(argv[argn], "%ld%c", &lval, &ch) < 1)
         usage('r');
      if (lval < 0 || lval > 65535L)
         usage('r');
      if (ch == 'b' || ch == 'B') 
      {
         cinfo->restart_interval = (unsigned int) lval;
         cinfo->restart_in_rows = 0; /* else prior '-restart n' overrides me */
      } else {
         cinfo->restart_in_rows = (int) lval;
         /* restart_interval will be computed during startup */
      }

    } 
    else 
    if (keymatch(arg, "sample", 2)) 
    {
      /* Set sampling factors. */
      if (++argn >= argc)         /* advance to next argument */
         usage('s');
      samplearg = argv[argn];
      /* Must delay setting sample factors until after we have processed any
       * colorspace-determining switches, since jpeg_set_colorspace sets
       * default sampling factors.
       */

    } 
    else 
    if (keymatch(arg, "scans", 2)) 
    {
      /* Set scan script. */
#ifdef C_MULTISCAN_FILES_SUPPORTED
      if (++argn >= argc)         /* advance to next argument */
         usage('s');
      scansarg = argv[argn];
      /* We must postpone reading the file in case -progressive appears. */
#else
      fprintf(stderr, "%s: sorry, multi-scan output was not compiled\n",
               lsProgName);
      PressAnyKey();
      exit(EXIT_FAILURE);
#endif

    } 
    else 
    if (keymatch(arg, "smooth", 2)
     || keymatch(arg, "smu", 3) ) 
    {
      /* Set input smoothing factor. */
      int val;

      if (++argn >= argc)         /* advance to next argument */
         usage('s');
      if (sscanf(argv[argn], "%d", &val) != 1)
         usage('s');
      if (val < 0 || val > 100)
         usage('s');
      cinfo->smoothing_factor = val;

    } 
    else 
    if (keymatch(arg, "targa", 3)) 
    {
      /* Input file is Targa format. */
      is_targa = TRUE;

    } 
    else 
    if (keymatch(arg, "help", 1) || !strcmp(arg, "-?") ) 
    {
      usage(0); 
    } 
    else    
    {
      fprintf(stderr, "%s: UNKNOWN KEYWORD %s\n",
               lsProgName, arg);
      usage(1);                           /* bad switch */
    }
  }

  /* Post-switch-scanning cleanup */

  if (for_real) 
  {

    /* Set quantization tables for selected quality. */
    /* Some or all may be overridden if -qtables is present. */
    jpeg_set_quality(cinfo, quality, force_baseline);

    if (qtablefile != NULL)         /* process -qtables if it was present */
      if (! read_quant_tables(cinfo, qtablefile,
                                 q_scale_factor, force_baseline))
         usage('q');

    if (qslotsarg != NULL)         /* process -qslots if it was present */
      if (! set_quant_slots(cinfo, qslotsarg))
         usage('q');

    if (samplearg != NULL)         /* process -sample if it was present */
      if (! set_sample_factors(cinfo, samplearg))
         usage('q');

#ifdef C_PROGRESSIVE_SUPPORTED
    if (simple_progressive)         /* process -progressive; -scans can override */
      jpeg_simple_progression(cinfo);
#endif

#ifdef C_MULTISCAN_FILES_SUPPORTED
    if (scansarg != NULL)         /* process -scans if it was present */
      if (! read_scan_script(cinfo, scansarg))
         usage('s');
#endif
  }

  return argn;                           /* return index of next arg (file name) */
}



#include <errno.h>

void FileErrorMsg()
{
  char *lsErrTxt;

  /*
  if (errno == ENOPATH)
      lsErrTxt = &"Path Not Found";
  else
  if (errno == ENOFILE)
      lsErrTxt = &"File Not Found";
  else
  */
  if (errno == ENOENT)
      lsErrTxt = &"Path or File Not Found";
  else
  if (errno == 13)
      lsErrTxt = &"File currently LOCKED by another task";
  else
  if (errno == EACCES)
      lsErrTxt = &"Security Access Denied";
  else
  if (errno == 1)
      lsErrTxt = &"BAD DISK SECTOR - Permanent Error";
  else
  if (errno == EBADF)
      lsErrTxt = &"Bad File Number";
  else
      lsErrTxt = &" ";

  fprintf(stderr, "   %s\n", lsErrTxt);
}





//* Check for overwrite of an existing file; clear it with user

#ifndef NO_OVERWRITE_CHECK
int iOverWrite_Chk = 1;

LOCAL(boolean)
is_overwrite_ok (char * outfname)
{
  FILE * ofile;
  int ch;

  ofile = fopen(outfname, READ_BINARY);
  if (ofile == NULL)
  {
     return TRUE;                  /* not present */
  }

  fclose(ofile);                  /* oops, it is present */

  for (;;) 
  {
    fprintf(stderr, "\rOutput File: %s%s already exists, overwrite it? [y/n/a/e] ",
             outfname, DSN_Spreader);
    fflush(stderr);

    ch = getche(stdin);
    fprintf(stderr, "\n");

    /*
    if (ch != '\n')                // flush rest of line 
      while (getc(stdin) != '\n')
         // nothing;
    */

    switch (ch) 
    {
    case 'A':
    case 'a':
      iOverWrite_Chk = 0;
    case 'Y':
    case 'y':
      return TRUE;

    // case '\n':
    case 'N':
    case 'n':
      return FALSE;

    // abort
    case 'E':
    case 'e':
    case 3:  // CTRL-C
        exit(EXIT_FAILURE);
    
    default:
      fprintf(stderr,"\nKeycode=%d\n", ch);
    /* otherwise, ask again */
    }

  }
}

#endif


char *lsDSN_NameBegin, *lsDSN_Dot, cDSN_LastChar;
void DSN_Analyse(char *lsParmFileName)
{
  char cCurr, *lcCurr, cLast;

  lcCurr          = lsParmFileName;
  lsDSN_NameBegin = lcCurr;
  lsDSN_Dot       = 0;
  cLast           = 0;

  for (;;)
  {
    cCurr = *lcCurr;
    if (cCurr == 0)
        break;

    if (cCurr == '.')
        lsDSN_Dot = lcCurr;
    else
    if (cCurr == ':' || cCurr == '\\')
        lsDSN_NameBegin = lcCurr+1;
 
    cLast = cCurr;
    lcCurr++;        
  }

  // If no dot found, point to end of string marker
  if (lsDSN_Dot == 0)
      lsDSN_Dot = lcCurr;

  iDSN_NameLen = lcCurr - lsParmFileName;
  if (iDSN_NameLen > 25)
       DSN_Spreader = &"\n    ";

  cDSN_LastChar = cLast;

  return;
}

/*  OLD CODE
    int i;
    for (i = strlen(lsOUT_FileName)-1; i >= 0; i--) 
    {
      switch (lsOUT_FileName[i]) 
      {
      case ':':
      case '/':
      case '\\':
         i = 0;                           // stop scanning 
         break;
      case '.':
         lsOUT_FileName[i] = '\0';         // lop off existing extension 
         i = 0;                           // stop scanning 
         break;
      default:
         break;                           // keep scanning 
      }
    }
    strcat(lsOUT_FileName, ".jpg");
}
*/



// Process a single input file name, and return its index in argv[].
// File names at or to left of old_file_index have been processed already.
//

LOCAL(int)
process_one_file (int argc, char **argv, int old_file_index)
{
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;

  char *lsIN_FileName;
  char lsWORK_FileName[PATH_MAX];
  int iErrorFlag;
  int iInSize, iOutSize;

#ifdef PROGRESS_REPORT
  struct cdjpeg_progress_mgr progress;
#endif
  int file_index;
  cjpeg_source_ptr src_mgr;
  FILE * IN_File = NULL;
  FILE * OUT_File = NULL;
  JDIMENSION num_scanlines;

  /* Initialize the JPEG compression object with default error handling. */
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_compress(&cinfo);
  /* Add some application-specific error messages (from cderror.h) */
  jerr.addon_message_table = cdjpeg_message_table;
  jerr.first_addon_message = JMSG_FIRSTADDONCODE;
  jerr.last_addon_message = JMSG_LASTADDONCODE;

  /* Now safe to enable signal catcher. */
#ifdef NEED_SIGNAL_CATCHER
  enable_signal_catcher((j_common_ptr) &cinfo);
#endif

  /* Initialize JPEG parameters.
   * Much of this may be overridden later.
   * In particular, we don't yet know the input file's color space,
   * but we need to provide some value for jpeg_set_defaults() to work.
   */

  cinfo.in_color_space = JCS_RGB; /* arbitrary guess */
  jpeg_set_defaults(&cinfo);

  /* Scan command line to find next file name.
   * It is convenient to use just one switch-parsing routine, but the switch
   * values read here are ignored; we will rescan the switches after opening
   * the input file.
   */

  file_index = parse_switches(&cinfo, argc, argv, old_file_index, FALSE);
  if (file_index >= argc) 
  {
    fprintf(stderr, "%s: missing input file name\n", lsProgName);
    usage(0);
  }

  /* Open the input file. */
  lsIN_FileName = argv[file_index];
  IN_File = fopen(lsIN_FileName, READ_BINARY);
  if (IN_File == NULL) 
  {
    fprintf(stderr, "%s: can't open %s\n", lsProgName, lsIN_FileName);
    FileErrorMsg();
    exit(EXIT_FAILURE);
  }
  else
    fprintf(stderr, "%s ...", lsIN_FileName);


#ifdef PROGRESS_REPORT
  start_progress_monitor((j_common_ptr) &cinfo, &progress);
#endif

  /* Figure out the input file format, and set up to read it. */
  src_mgr = select_file_type(&cinfo, IN_File);
  src_mgr->input_file = IN_File;

  /* Read the input file header to obtain file size & colorspace. */
  (*src_mgr->start_input) (&cinfo, src_mgr);

  /* Now that we know input colorspace, fix colorspace-dependent defaults */
  jpeg_default_colorspace(&cinfo);

  /* Adjust default compression parameters by re-parsing the options */
  file_index = parse_switches(&cinfo, argc, argv, old_file_index, TRUE);

  // We may need to generate the output file or path or extension 

  /* If user didn't supply -outfile switch, generate output file name. */
  if (lsOUT_FileName == NULL) 
  {
    /* Make lsOUT_FileName be lsIN_FileName with .jpg substituted for extension */

    strcpy(lsWORK_FileName, lsIN_FileName);

    // set output type to jpg
    DSN_Analyse(lsWORK_FileName);
    strcpy(lsDSN_Dot, ".jpg");

    lsOUT_FileName = lsWORK_FileName;
  }
  else
  {
    strcpy(lsWORK_FileName, lsOUT_FileName);
    DSN_Analyse(lsWORK_FileName);
    if (cDSN_LastChar == '\\')
    {
        // Outfile name is a path spec
        cOUT_Wild_Flag = 1;


        // Keep drive:path
        lsOUT_NameBegin = lsDSN_NameBegin;
        
        // check accessibility of output folder
        *lsOUT_NameBegin = 0;
        iErrorFlag = access(lsWORK_FileName, 2);
        if (iErrorFlag)
        {
            fprintf(stderr, "\nCannot access output folder. Err=%d\nOut= %s\n", 
                                            errno, lsWORK_FileName);
            FileErrorMsg();
            exit(EXIT_FAILURE);
        }

        // copy File Name.Ext from input 
        DSN_Analyse(lsIN_FileName);
        strcpy (lsOUT_NameBegin, lsDSN_NameBegin);

        // set output type to jpg
        DSN_Analyse(lsWORK_FileName);
        strcpy(lsDSN_Dot, ".jpg");

        lsOUT_FileName = lsWORK_FileName;
    }
    else
      cOUT_Wild_Flag = 0;
  }


  if (!stricmp(lsIN_FileName, lsOUT_FileName))
  {
      fprintf(stderr, "\rSAME INPUT AND OUTPUT FILE NAMES.\nFile or folder names or extension need to be different\n In= %s\nOut= %s\n", 
                                            lsIN_FileName, lsOUT_FileName);
      goto fail;
  }

  fprintf(stderr, "\rJPEGing %s%s => %s\n", 
                     lsIN_FileName, DSN_Spreader, 
                     lsOUT_FileName);

#ifndef NO_OVERWRITE_CHECK
  if (iOverWrite_Chk)
  {
    if(! is_overwrite_ok(lsOUT_FileName))
       goto fail;
  }
#endif

  /* Open the output file. */
  if ((OUT_File = fopen(lsOUT_FileName, WRITE_BINARY)) == NULL) 
  {
    fprintf(stderr, "%s: can't create %s\n", lsProgName, lsOUT_FileName);
    FileErrorMsg();
    goto fail;
  }

  /* Specify data destination for compression */
  jpeg_stdio_dest(&cinfo, OUT_File);

  /* Start compressor */
  jpeg_start_compress(&cinfo, TRUE);

  /* Process data */
  while (cinfo.next_scanline < cinfo.image_height) 
  {
    num_scanlines = (*src_mgr->get_pixel_rows) (&cinfo, src_mgr);
    (void) jpeg_write_scanlines(&cinfo, src_mgr->buffer, num_scanlines);
  }

  /* Finish compression and release memory */
  (*src_mgr->finish_input) (&cinfo, src_mgr);
  jpeg_finish_compress(&cinfo);

  if (IN_File != stdin)
      iInSize = (ftell(IN_File) +1023) / 1024;
  if (OUT_File != stdout)
      iOutSize = (ftell(OUT_File) +1023) / 1024;

  fprintf(stderr, "   %ik -> %ik\n",
                      iInSize, iOutSize);

  /* Clean up and exit */
fail:
  jpeg_destroy_compress(&cinfo);

  if (IN_File != NULL) fclose(IN_File);
  if (OUT_File != NULL) fclose(OUT_File);

#ifdef PROGRESS_REPORT
  end_progress_monitor((j_common_ptr) &cinfo);
#endif
 
  /* Disable signal catcher. */
#ifdef NEED_SIGNAL_CATCHER
  enable_signal_catcher((j_common_ptr) NULL);
#endif

  return file_index;
}





//* The main program.
 

int
main (int argc, char **argv)
{
  int file_index;

  cOUT_Wild_Flag = 0;


  /* On Mac, fetch a command line. */
#ifdef USE_CCOMMAND
  argc = ccommand(&argv);
#endif

#ifdef MSDOS
  lsProgName = "cjpeg";                  /* DOS tends to be too verbose about argv[0] */
#else
  lsProgName = argv[0];
  if (lsProgName == NULL || lsProgName[0] == 0)
    lsProgName = "cjpeg";                  /* in case C library doesn't provide it */
#endif

  /* The default maxmem must be computed only once at program startup,
   * since releasing memory with free() won't give it back to the OS.
   */
#ifdef FREE_MEM_ESTIMATE
  default_maxmem = FREE_MEM_ESTIMATE;
#else
  default_maxmem = 0;
#endif

  /* Scan command line, parse switches and locate input file names */

  if (argc < 2)
    usage(0);                           /* nothing on the command line?? */

  file_index = 0;

  while (file_index < argc-1)
    file_index = process_one_file(argc, argv, file_index);

  /* All done. */
  exit(EXIT_SUCCESS);
  return 0;                           /* suppress no-return-value warnings */
}
