/*
 * alternate djpeg.c
 *
 * Copyright (C) 1991-1997, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file contains an alternate user interface for the JPEG decompressor.
 * One or more input files are named on the command line, and output file
 * names are created by substituting an appropriate extension.
 */

#include "cdjpeg.h"        /* Common decls for cjpeg/djpeg applications */
#include "jversion.h"        /* for version message */

#include <ctype.h>        /* to declare isprint() */

#ifdef USE_CCOMMAND        /* command-line reader for Macintosh */
#ifdef __MWERKS__
#include <SIOUX.h>              /* Metrowerks needs this */
#include <console.h>        /* ... and this */
#endif
#ifdef THINK_C
#include <console.h>        /* Think declares it here */
#endif
#endif

#ifndef PATH_MAX        /* ANSI maximum-pathname-length constant */
#define PATH_MAX 256
#endif


/* Create the add-on message string table. */

#define JMESSAGE(code,string)    string ,

static const char * const cdjpeg_message_table[] = {
#include "cderror.h"
  NULL
};


/*
 * Automatic determination of available memory.
 */

static long default_maxmem;    /* saves value determined at startup, or 0 */

#ifndef FREE_MEM_ESTIMATE    /* may be defined from command line */

#ifdef MSDOS            /* For MS-DOS (unless flat-memory model) */

#include <dos.h>        /* for access to intdos() call */

LOCAL(long)
unused_dos_memory (void)
/* Obtain total amount of unallocated DOS memory */
{
  union REGS regs;
  long nparas;

  regs.h.ah = 0x48;        /* DOS function Allocate Memory Block */
  regs.x.bx = 0xFFFF;        /* Ask for more memory than DOS can have */
  (void) intdos(&regs, &regs);
  /* DOS will fail and return # of paragraphs actually available in BX. */
  nparas = (unsigned int) regs.x.bx;
  /* Times 16 to convert to bytes. */
  return nparas << 4;
}

/* The default memory setting is 95% of the available space. */
#define FREE_MEM_ESTIMATE  ((unused_dos_memory() * 95L) / 100L)

#endif /* MSDOS */

#ifdef ATARI            /* For Atari ST/STE/TT, Pure C or Turbo C */

#include <ext.h>

/* The default memory setting is 90% of the available space. */
#define FREE_MEM_ESTIMATE  (((long) coreleft() * 90L) / 100L)

#endif /* ATARI */

/* Add memory-estimation procedures for other operating systems here,
 * with appropriate #ifdef's around them.
 */

#endif /* !FREE_MEM_ESTIMATE */


/*
 * This list defines the known output image formats
 * (not all of which need be supported by a given version).
 * You can change the default output format by defining DEFAULT_FMT;
 * indeed, you had better do so if you undefine PPM_SUPPORTED.
 */

typedef enum {
    FMT_BMP,        /* BMP format (Windows flavor) */
    FMT_GIF,        /* GIF format */
    FMT_OS2,        /* BMP format (OS/2 flavor) */
    FMT_PPM,        /* PPM/PGM (PBMPLUS formats) */
    FMT_RLE,        /* RLE format */
    FMT_TARGA,        /* Targa format */
    FMT_TIFF        /* TIFF format */
} IMAGE_FORMATS;

#ifndef DEFAULT_FMT        /* so can override from CFLAGS in Makefile */
#define DEFAULT_FMT    FMT_BMP
#endif

static IMAGE_FORMATS requested_fmt;


/*
 * Argument-parsing code.
 * The switch parser is designed to be useful with DOS-style command line
 * syntax, ie, intermixed switches and file names, where only the switches
 * to the left of a given file name affect processing of that file.
 */


static const char * lsProgName;    /* program name for error messages */
static char * lsOUT_FileName;    /* for -outfile switch */
static char *lsOUT_NameBegin;
int iDSN_NameLen = 0;
char *DSN_Spreader = &" ";

void PressAnyKey()
{
  char cT;
  fprintf(stderr, "                      --- press any key ---");
  getche(cT);
  fprintf(stderr, "\n");
}


LOCAL(void) usage (char cP1)
/* complain about bad command line */
{
  if (cP1 == 0)
  {
    fprintf(stderr, "usage: %s [switches] inputfile(s)\n", lsProgName);
    fprintf(stderr, "List of input files may use wildcards (* and ?)\n");
    fprintf(stderr, "Output filename defaults to input filename but with .jpg extension\n\n");
    fprintf(stderr, "Switches must PRECEED inputfile names.\n");
  }

  if (cP1 == 'o' || cP1 < 2)
  {
     fprintf(stderr, "  -Outfile name  Must preceed Input name. Folder must end in slash.\n");
  }

  if (cP1 == 'c' || cP1 < 2)
  {
    fprintf(stderr, "  -colors N      Reduce image to no more than N colors\n");
  }

//  if (cP1 == 'f' || cP1 < 2)
//  {
//    fprintf(stderr, "  -fast          Fast, low-quality processing\n");
//  }

  if (cP1 == 'g' || cP1 < 2)
  {
    fprintf(stderr, "  -grayscale     Force grayscale output\n");
  }

#ifdef IDCT_SCALING_SUPPORTED
  if (cP1 == 's' || cP1 < 2)
  {
    fprintf(stderr, "  -scale M/N     Scale output image by fraction M/N, eg, 1/8\n");
  }
#endif
#ifdef BMP_SUPPORTED
  if (cP1 == 'b' || cP1 < 2)
  {
     fprintf(stderr, "  -bmp           Select BMP output format (Windows style)%s\n",
                (DEFAULT_FMT == FMT_BMP ? " (default)" : ""));
  }
#endif

/*
#ifdef GIF_SUPPORTED
  fprintf(stderr, "  -gif           Select GIF output format%s\n",
      (DEFAULT_FMT == FMT_GIF ? " (default)" : ""));
#endif
*/
#ifdef BMP_SUPPORTED
  if (cP1 == 'z' || cP1 < 2)
  {
    fprintf(stderr, "  -os2           Select BMP output format (OS/2 style)%s\n",
          (DEFAULT_FMT == FMT_OS2 ? " (default)" : ""));
  }
#endif
#ifdef PPM_SUPPORTED
  if (cP1 == 'y' || cP1 < 2)
  {
     fprintf(stderr, "  -pnm           Select PBMPLUS (PPM/PGM) output format%s\n",
          (DEFAULT_FMT == FMT_PPM ? " (default)" : ""));
  }
#endif
#ifdef RLE_SUPPORTED
  if (cP1 == 'x' || cP1 < 2)
  {
     fprintf(stderr, "  -rle           Select Utah RLE output format%s\n",
        (DEFAULT_FMT == FMT_RLE ? " (default)" : ""));
  }
#endif
#ifdef TARGA_SUPPORTED
  if (cP1 == 't' || cP1 < 2)
  {
     fprintf(stderr, "  -targa         Select Targa output format%s\n",
        (DEFAULT_FMT == FMT_TARGA ? " (default)" : ""));
  }
#endif
  if (cP1 < 2)
  {
     fprintf(stderr, "Switches for advanced users:\n");
  }
  if (cP1 < 2)
  {
     fprintf(stderr, "  -dct [Int|Fast|Float]  DCT method.\n");
  }
#ifdef DCT_ISLOW_SUPPORTED
  if (cP1 == 'd')
  {
     fprintf(stderr, "  -dct int       Use integer DCT method%s\n",
      (JDCT_DEFAULT == JDCT_ISLOW ? " (default)" : ""));
  }
#endif
#ifdef DCT_IFAST_SUPPORTED
  if (cP1 == 'd')
  {
     fprintf(stderr, "  -dct fast      Use fast integer DCT (less accurate)%s\n",
        (JDCT_DEFAULT == JDCT_IFAST ? " (default)" : ""));
  }
#endif
#ifdef DCT_FLOAT_SUPPORTED
  if (cP1 == 'd')
  {
     fprintf(stderr, "  -dct float     Use floating-point DCT method%s\n",
         (JDCT_DEFAULT == JDCT_FLOAT ? " (default)" : ""));
  }
#endif


  if (cP1 == 'h' || cP1 < 2)
  {
     fprintf(stderr, "  -dither [Fs|None|Ordered] Ditherhing in quantization\n");
  }
  else
  if (cP1 == 'h' || cP1 < 2)
  {
     fprintf(stderr, "  -dither fs       Use F-S dithering (default)\n");
     fprintf(stderr, "  -dither none     Don't use dithering in quantization\n");
     fprintf(stderr, "  -dither ordered  Use ordered dither (medium speed, quality)\n");
  }



#ifdef QUANT_2PASS_SUPPORTED
  if (cP1 == 'f' || cP1 < 2)
  {
     fprintf(stderr, "  -map FILE      Map to colors used in named image file\n");
  }
#endif
  if (cP1 == 's' || cP1 < 2)
  {
     fprintf(stderr, "  -nosmooth      Any upsampling will not be high-quality\n");
  }

//#ifdef QUANT_1PASS_SUPPORTED
//  if (cP1 == 'p' || cP1 < 2)
//  {
//     fprintf(stderr, "  -onepass       Use 1-pass quantization (fast, low quality)\n");
//  }
//#endif

#ifndef FREE_MEM_ESTIMATE
  if (cP1 == 'm' || cP1 < 2)
  {
     fprintf(stderr, "  -maxmemory N   Maximum memory to use (in kbytes)\n");
  }
#endif

  if (cP1 == 'v' || cP1 < 2)
  {
     fprintf(stderr, "  -verbose  or  -debug   Emit debug output\n");
  }


  PressAnyKey();
  exit(EXIT_FAILURE);
}


char cOUT_Wild_Flag;


LOCAL(int)
parse_switches (j_decompress_ptr cinfo, int argc, char **argv,
        int last_file_arg_seen, boolean for_real)
/* Parse optional switches.
 * Returns argv[] index of first file-name argument (== argc if none).
 * Any file names with indexes <= last_file_arg_seen are ignored;
 * they have presumably been processed in a previous iteration.
 * (Pass 0 for last_file_arg_seen on the first or only iteration.)
 * for_real is FALSE on the first (dummy) pass; we may skip any expensive
 * processing.
 */
{
  int argn;
  char * arg;

  /* Set up default JPEG parameters. */
  requested_fmt = DEFAULT_FMT;    /* set default output file format */
  lsOUT_FileName = NULL;
  cinfo->err->trace_level = 0;
  if (default_maxmem > 0)    /* override library's default value */
    cinfo->mem->max_memory_to_use = default_maxmem;

  /* Scan command line options, adjust parameters */

  for (argn = 1; argn < argc; argn++) 
  {
    arg = argv[argn];
    if (*arg != '-') 
    {
      /* Not a switch, must be a file name argument */
      if (argn <= last_file_arg_seen) 
      {
          if (!cOUT_Wild_Flag)
              lsOUT_FileName = NULL;  // -outfile applies to just one input file 
          continue;        /* ignore this name if previously processed */
      }
      break;            /* else done parsing switches */
    }
    arg++;            /* advance past switch marker character */

    if (keymatch(arg, "bmp", 1)) 
    {
      /* BMP output format. */
      requested_fmt = FMT_BMP;

    } 
    else 
    if (keymatch(arg, "colors", 1) || keymatch(arg, "colours", 1) ||
           keymatch(arg, "quantize", 1) || keymatch(arg, "quantise", 1)) 
    {
      /* Do color quantization. */
      int val;

      if (++argn >= argc)    /* advance to next argument */
         usage(0);
      if (sscanf(argv[argn], "%d", &val) != 1)
         usage(0);

      cinfo->desired_number_of_colors = val;
      cinfo->quantize_colors = TRUE;

    } 
    else 
    if (keymatch(arg, "dct", 2)) 
    {
      /* Select IDCT algorithm. */
      if (++argn >= argc)    /* advance to next argument */
          usage('d');
      if (keymatch(argv[argn], "int", 1)) 
      {
          cinfo->dct_method = JDCT_ISLOW;
      }
      else 
      if (keymatch(argv[argn], "fast", 2)) 
      {
          cinfo->dct_method = JDCT_IFAST;
      } 
      else 
      if (keymatch(argv[argn], "float", 2)) 
      {
          cinfo->dct_method = JDCT_FLOAT;
      } 
      else
          usage('d');

    } 
    else 
    if (keymatch(arg, "dither", 2)) 
    {
      /* Select dithering algorithm. */
      if (++argn >= argc)    /* advance to next argument */
          usage('h');
      if (keymatch(argv[argn], "fs", 2)) 
      {
          cinfo->dither_mode = JDITHER_FS;
      } 
      else 
      if (keymatch(argv[argn], "none", 2)) 
      {
          cinfo->dither_mode = JDITHER_NONE;
      } 
      else 
      if (keymatch(argv[argn], "ordered", 2)) 
      {
          cinfo->dither_mode = JDITHER_ORDERED;
      } 
      else
          usage('h');

    } 
    else 
    if (keymatch(arg, "debug", 1) 
     || keymatch(arg, "dbg", 1)
     || keymatch(arg, "verbose", 1)) 
    {
      /* Enable debug printouts. */
      /* On first -d, print version identification */
      static boolean printed_version = FALSE;

      if (! printed_version) 
      {
             fprintf(stderr, "Independent JPEG Group's DJPEG, version %s\n%s\n",
                           JVERSION, JCOPYRIGHT);
             printed_version = TRUE;
      }
      cinfo->err->trace_level++;

    } 
    else 
    if (keymatch(arg, "fast", 1)) 
    {
      /* Select recommended processing options for quick-and-dirty output. */
      cinfo->two_pass_quantize = FALSE;
      cinfo->dither_mode = JDITHER_ORDERED;

      if (! cinfo->quantize_colors) /* don't override an earlier -colors */
            cinfo->desired_number_of_colors = 216;

      cinfo->dct_method = JDCT_FASTEST;
      cinfo->do_fancy_upsampling = FALSE;

    } 
    else 
    if (keymatch(arg, "gif", 1)) 
    {
      /* GIF output format. */
      requested_fmt = FMT_GIF;

    } 
    else 
    if (keymatch(arg, "grayscale", 2) || keymatch(arg, "greyscale",2)) 
    {
      /* Force monochrome output. */
      cinfo->out_color_space = JCS_GRAYSCALE;

    } 
    else 
    if (keymatch(arg, "map", 3)) 
    {
      /* Quantize to a color map taken from an input file. */
      if (++argn >= argc)    /* advance to next argument */
          usage('f');

      if (for_real) 
      {        /* too expensive to do twice! */
#ifdef QUANT_2PASS_SUPPORTED    /* otherwise can't quantize to supplied map */
          FILE * mapfile;

         if ((mapfile = fopen(argv[argn], READ_BINARY)) == NULL) 
         {
             fprintf(stderr, "%s: can't open %s\n", lsProgName, argv[argn]);
             exit(EXIT_FAILURE);
         }
        read_color_map(cinfo, mapfile);
        fclose(mapfile);
        cinfo->quantize_colors = TRUE;
#else
        ERREXIT(cinfo, JERR_NOT_COMPILED);
#endif
      }

    } 
    else 
    if (keymatch(arg, "maxmemory", 3)) 
    {
      /* Maximum memory in Kb (or Mb with 'm'). */
      long lval;
      char ch = 'x';

      if (++argn >= argc)    /* advance to next argument */
         usage('m');
      if (sscanf(argv[argn], "%ld%c", &lval, &ch) < 1)
          usage('m');
      if (ch == 'm' || ch == 'M')
         lval *= 1000L;
      cinfo->mem->max_memory_to_use = lval * 1000L;

    } 
    else 
    if (keymatch(arg, "nosmooth", 3)) 
    {
      /* Suppress fancy upsampling */
      cinfo->do_fancy_upsampling = FALSE;

    } 
    else 
    if (keymatch(arg, "onepass", 3)) 
    {
      /* Use fast one-pass quantization. */
      cinfo->two_pass_quantize = FALSE;

    } 
    else 
    if (keymatch(arg, "os2", 3)) 
    {
      /* BMP output format (OS/2 flavor). */
      requested_fmt = FMT_OS2;

    } 
    else 
    if (keymatch(arg, "outfile", 1)) 
    {
      /* Set output file name. */
      if (++argn >= argc)    /* advance to next argument */
          usage('o');
      lsOUT_FileName = argv[argn];    /* save it away for later use */

    } 
    else 
    if (keymatch(arg, "pnm", 1) 
     || keymatch(arg, "ppm", 1)) 
    {
      /* PPM/PGM output format. */
      requested_fmt = FMT_PPM;

    } 
    else 
    if (keymatch(arg, "rle", 1)) 
    {
      /* RLE output format. */
      requested_fmt = FMT_RLE;

    } 
    else 
    if (keymatch(arg, "scale", 1)) 
    {
      /* Scale the output image by a fraction M/N. */
      if (++argn >= argc)    /* advance to next argument */
          usage('s');
      if (sscanf(argv[argn], "%d/%d",
                 &cinfo->scale_num, &cinfo->scale_denom) != 2)
         usage('s');

    } 
    else 
    if (keymatch(arg, "targa", 1)) 
    {
      /* Targa output format. */
      requested_fmt = FMT_TARGA;

    } 
    else 
    {
      usage(0);            /* bogus switch */
    }
  }

  return argn;            /* return index of next arg (file name) */
}


/*
 * Marker processor for COM and interesting APPn markers.
 * This replaces the library's built-in processor, which just skips the marker.
 * We want to print out the marker as text, to the extent possible.
 * Note this code relies on a non-suspending data source.
 */

LOCAL(unsigned int)
jpeg_getc (j_decompress_ptr cinfo)
/* Read next byte */
{
  struct jpeg_source_mgr * datasrc = cinfo->src;

  if (datasrc->bytes_in_buffer == 0) {
    if (! (*datasrc->fill_input_buffer) (cinfo))
      ERREXIT(cinfo, JERR_CANT_SUSPEND);
  }
  datasrc->bytes_in_buffer--;
  return GETJOCTET(*datasrc->next_input_byte++);
}


METHODDEF(boolean)
print_text_marker (j_decompress_ptr cinfo)
{
  boolean traceit = (cinfo->err->trace_level >= 1);
  INT32 length;
  unsigned int ch;
  unsigned int lastch = 0;

  length = jpeg_getc(cinfo) << 8;
  length += jpeg_getc(cinfo);
  length -= 2;            /* discount the length word itself */

  if (traceit) {
    if (cinfo->unread_marker == JPEG_COM)
      fprintf(stderr, "Comment, length %ld:\n", (long) length);
    else            /* assume it is an APPn otherwise */
      fprintf(stderr, "APP%d, length %ld:\n",
          cinfo->unread_marker - JPEG_APP0, (long) length);
  }

  while (--length >= 0) {
    ch = jpeg_getc(cinfo);
    if (traceit) {
      /* Emit the character in a readable form.
       * Nonprintables are converted to \nnn form,
       * while \ is converted to \\.
       * Newlines in CR, CR/LF, or LF form will be printed as one newline.
       */
      if (ch == '\r') {
    fprintf(stderr, "\n");
      } else if (ch == '\n') {
    if (lastch != '\r')
      fprintf(stderr, "\n");
      } else if (ch == '\\') {
    fprintf(stderr, "\\\\");
      } else if (isprint(ch)) {
    putc(ch, stderr);
      } else {
    fprintf(stderr, "\\%03o", ch);
      }
      lastch = ch;
    }
  }

  if (traceit)
    fprintf(stderr, "\n");

  return TRUE;
}



#include <errno.h>

void FileErrorMsg()
{
  char *lsErrTxt;

  /*
  if (errno == ENOPATH)
      lsErrTxt = &"Path Not Found";
  else
  if (errno == ENOFILE)
      lsErrTxt = &"File Not Found";
  else
  */
  if (errno == ENOENT)
      lsErrTxt = &"Path or File Not Found";
  else
  if (errno == 13)
      lsErrTxt = &"File currently LOCKED by another task";
  else
  if (errno == EACCES)
      lsErrTxt = &"Security Access Denied";
  else
  if (errno == 1)
      lsErrTxt = &"BAD DISK SECTOR - Permanent Error";
  else
  if (errno == EBADF)
      lsErrTxt = &"Bad File Number";
  else
      lsErrTxt = &" ";

  fprintf(stderr, "   %s\n", lsErrTxt);
}




/*
 * Check for overwrite of an existing file; clear it with user
 */

#ifndef NO_OVERWRITE_CHECK

int iOverWrite_Chk = 1;

LOCAL(boolean) is_overwrite_ok (char * outfname)
{
  FILE * ofile;
  int ch;

  ofile = fopen(outfname, READ_BINARY);
  if (ofile == NULL)
  {
     return TRUE;                  /* not present */
  }

  fclose(ofile);                  /* oops, it is present */

  for (;;) 
  {
    fprintf(stderr, "\rOutput File: %s%s already exists, overwrite it? [y/n/a/e] ",
             outfname, DSN_Spreader);
    fflush(stderr);

    ch = getche(stdin);
    fprintf(stderr, "\n");

    /*
    if (ch != '\n')                // flush rest of line 
      while (getc(stdin) != '\n')
         // nothing;
    */

    switch (ch) 
    {
    case 'A':
    case 'a':
      iOverWrite_Chk = 0;
    case 'Y':
    case 'y':
      return TRUE;

    // case '\n':
    case 'N':
    case 'n':
      return FALSE;

    // abort
    case 'E':
    case 'e':
    case 3:  // CTRL-C
        exit(EXIT_FAILURE);
    
    default:
      fprintf(stderr,"\nKeycode=%d\n", ch);
    /* otherwise, ask again */
    }

  }
}

#endif


char *lsDSN_NameBegin, *lsDSN_Dot, cDSN_LastChar;
void DSN_Analyse(char *lsParmFileName)
{
  char cCurr, *lcCurr, cLast;

  lcCurr          = lsParmFileName;
  lsDSN_NameBegin = lcCurr;
  lsDSN_Dot       = 0;
  cLast           = 0;

  for (;;)
  {
    cCurr = *lcCurr;
    if (cCurr == 0)
        break;

    if (cCurr == '/')
    {  
        cCurr   = '\\';
        *lcCurr = '\\';
    }

    if (cCurr == '.')
        lsDSN_Dot = lcCurr;
    else
    if (cCurr == ':' || cCurr == '\\')
        lsDSN_NameBegin = lcCurr+1;
 
    cLast = cCurr;
    lcCurr++;        
  }

  // If no dot found, point to end of string marker
  if (lsDSN_Dot == 0)
      lsDSN_Dot = lcCurr;

  iDSN_NameLen = lcCurr - lsParmFileName;
  if (iDSN_NameLen > 25)
       DSN_Spreader = &"\n    ";

  cDSN_LastChar = cLast;

  return;
}


/*
 * Process a single input file name, and return its index in argv[].
 * File names at or to left of old_file_index have been processed already.
 */

LOCAL(int)
process_one_file (int argc, char **argv, int old_file_index)
{
  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr jerr;
  char *lsIN_FileName;
  char lsWORK_FileName[PATH_MAX];
  int iErrorFlag;
  int iInSize, iOutSize;

  const char *lsDefault_Extension = NULL;
#ifdef PROGRESS_REPORT
  struct cdjpeg_progress_mgr progress;
#endif
  int file_index;
  djpeg_dest_ptr dest_mgr = NULL;
  FILE * input_file = NULL;
  FILE * output_file = NULL;
  JDIMENSION num_scanlines;

  /* Initialize the JPEG decompression object with default error handling. */
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_decompress(&cinfo);
  /* Add some application-specific error messages (from cderror.h) */
  jerr.addon_message_table = cdjpeg_message_table;
  jerr.first_addon_message = JMSG_FIRSTADDONCODE;
  jerr.last_addon_message = JMSG_LASTADDONCODE;

  /* Insert custom marker processor for COM and APP12.
   * APP12 is used by some digital camera makers for textual info,
   * so we provide the ability to display it as text.
   * If you like, additional APPn marker types can be selected for display,
   * but don't try to override APP0 or APP14 this way (see libjpeg.doc).
   */
  jpeg_set_marker_processor(&cinfo, JPEG_COM, print_text_marker);
  jpeg_set_marker_processor(&cinfo, JPEG_APP0+12, print_text_marker);

  /* Now safe to enable signal catcher. */
#ifdef NEED_SIGNAL_CATCHER
  enable_signal_catcher((j_common_ptr) &cinfo);
#endif

  /* Scan command line to find next file name.
   * It is convenient to use just one switch-parsing routine, but the switch
   * values read here are ignored; we will rescan the switches after opening
   * the input file.
   * (Exception: tracing level set here controls verbosity for COM markers
   * found during jpeg_read_header...)
   */

  file_index = parse_switches(&cinfo, argc, argv, old_file_index, FALSE);
  if (file_index >= argc) 
  {
    fprintf(stderr, "%s: missing input file name\n", lsProgName);
    usage(3);
  }

  /* Open the input file. */
  lsIN_FileName = argv[file_index];
  if ((input_file = fopen(lsIN_FileName, READ_BINARY)) == NULL) 
  {
    fprintf(stderr, "%s: can't open %s\n", lsProgName, lsIN_FileName);
    FileErrorMsg();
    goto fail;
  }

#ifdef PROGRESS_REPORT
  start_progress_monitor((j_common_ptr) &cinfo, &progress);
#endif

  /* Specify data source for decompression */
  jpeg_stdio_src(&cinfo, input_file);

  /* Adjust default decompression parameters by re-parsing the options */
  file_index = parse_switches(&cinfo, argc, argv, old_file_index, TRUE);
 
  /* Read file header, set default decompression parameters */
  (void) jpeg_read_header(&cinfo, TRUE);


  /* Initialize the output module now to let it override any crucial
   * option settings (for instance, GIF wants to force color quantization).
   */ 
  switch (requested_fmt) 
  {  
#ifdef BMP_SUPPORTED
  case FMT_BMP:
    dest_mgr = jinit_write_bmp(&cinfo, FALSE);
    lsDefault_Extension = ".bmp";
    break;
  case FMT_OS2:
    dest_mgr = jinit_write_bmp(&cinfo, TRUE);
    lsDefault_Extension = ".bmp";
    break;
#endif
#ifdef GIF_SUPPORTED
  case FMT_GIF:
    dest_mgr = jinit_write_gif(&cinfo);
    lsDefault_Extension = ".gif";
    break;
#endif
#ifdef PPM_SUPPORTED
  case FMT_PPM:
    dest_mgr = jinit_write_ppm(&cinfo);
    lsDefault_Extension = ".ppm";
    break;
#endif
#ifdef RLE_SUPPORTED
  case FMT_RLE:
    dest_mgr = jinit_write_rle(&cinfo);
    lsDefault_Extension = ".rle";
    break;
#endif
#ifdef TARGA_SUPPORTED
  case FMT_TARGA:
    dest_mgr = jinit_write_targa(&cinfo);
    lsDefault_Extension = ".tga";
    break;
#endif
  default:
    ERREXIT(&cinfo, JERR_UNSUPPORTED_FORMAT);
    break;
  }

  /* If user didn't supply -outfile switch, select output file name. */
  if (lsOUT_FileName == 0  
  ||  lsOUT_FileName == NULL
  || *lsOUT_FileName == 0  ) 
  {
    //int i;

    strcpy(lsWORK_FileName, lsIN_FileName);

    // set output type to jpg
    DSN_Analyse(lsWORK_FileName);
    strcpy(lsDSN_Dot, lsDefault_Extension);

    lsOUT_FileName = lsWORK_FileName;

    /*  OLD_CODE 
    lsOUT_FileName = lsWORK_FileName;

    // Make lsOUT_FileName be lsIN_FileName with appropriate extension 

    strcpy(lsOUT_FileName, lsIN_FileName);
    for (i = strlen(lsOUT_FileName)-1; i >= 0; i--) 
    {
      switch (lsOUT_FileName[i]) {
      case ':':
      case '/':
      case '\\':
    i = 0;            // stop scanning 
    break;
      case '.':
    lsOUT_FileName[i] = '\0';    // lop off existing extension 
    i = 0;            // stop scanning 
    break;
      default:
    break;            // keep scanning 
      }
    }
    strcat(lsOUT_FileName, lsDefault_Extension);
    */
  }
  else
  {
    strcpy(lsWORK_FileName, lsOUT_FileName);
    DSN_Analyse(lsWORK_FileName);
    if (cDSN_LastChar == '\\')
    {
        // Outfile name is a path spec
        cOUT_Wild_Flag = 1;

        // Keep drive:path
        lsOUT_NameBegin = lsDSN_NameBegin;
        
        // check accessibility of output folder
        *lsOUT_NameBegin = 0;
        iErrorFlag = access(lsWORK_FileName, 2);
        if (iErrorFlag)
        {
            fprintf(stderr, "\rCannot access output folder. Err=%d\nOut= %s\n", 
                                            errno, lsWORK_FileName);
            FileErrorMsg();
            goto fail; 
        }

        // copy File Name.Ext from input 
        DSN_Analyse(lsIN_FileName);
        strcpy (lsOUT_NameBegin, lsDSN_NameBegin);

        // set output type to jpg
        DSN_Analyse(lsWORK_FileName);
        strcpy(lsDSN_Dot, lsDefault_Extension);

        lsOUT_FileName = lsWORK_FileName;
    }
    else
      cOUT_Wild_Flag = 0;
  }

  if (!stricmp(lsIN_FileName, lsOUT_FileName))
  {
      fprintf(stderr, "\rSAME INPUT AND OUTPUT FILE NAMES.\nFile or folder names or extension need to be different\n In= %s\nOut= %s\n", 
                                            lsIN_FileName, lsOUT_FileName);
      goto fail;
  }

  fprintf(stderr, "\rDecompressing %s%s => %s\n", 
                     lsIN_FileName, DSN_Spreader, 
                     lsOUT_FileName);
  //fprintf(stderr, "Decompressing %s => %s\n", lsIN_FileName, lsOUT_FileName);

#ifndef NO_OVERWRITE_CHECK
  if (! is_overwrite_ok(lsOUT_FileName))
    goto fail;
#endif

  /* Open the output file. */
  if ((output_file = fopen(lsOUT_FileName, WRITE_BINARY)) == NULL) 
  {
    fprintf(stderr, "%s: can't create %s\n", lsProgName, lsOUT_FileName);
    goto fail;
  }
  dest_mgr->output_file = output_file;

  /* Start decompressor */
  (void) jpeg_start_decompress(&cinfo);


  /* Write output file header */
  (*dest_mgr->start_output) (&cinfo, dest_mgr);

  /* Process data */
  while (cinfo.output_scanline < cinfo.output_height) {
    num_scanlines = jpeg_read_scanlines(&cinfo, dest_mgr->buffer,
                    dest_mgr->buffer_height);
    (*dest_mgr->put_pixel_rows) (&cinfo, dest_mgr, num_scanlines);
  }

#ifdef PROGRESS_REPORT
  /* Hack: count final pass as done in case finish_output does an extra pass.
   * The library won't have updated completed_passes.
   */
  progress.pub.completed_passes = progress.pub.total_passes;
#endif

  /* Finish decompression and release memory.
   * I must do it in this order because output module has allocated memory
   * of lifespan JPOOL_IMAGE; it needs to finish before releasing memory.
   */
  (*dest_mgr->finish_output) (&cinfo, dest_mgr);
  (void) jpeg_finish_decompress(&cinfo);

  if (input_file != stdin)
      iInSize = (ftell(input_file) +1023) / 1024;
  if (output_file != stdout)
      iOutSize = (ftell(output_file) +1023) / 1024;

  fprintf(stderr, "  %ik -> %ik\n",
                      iInSize, iOutSize);

  /* Clean up and exit */
fail:
  jpeg_destroy_decompress(&cinfo);

  if (input_file != NULL) fclose(input_file);
  if (output_file != NULL) fclose(output_file);

#ifdef PROGRESS_REPORT
  end_progress_monitor((j_common_ptr) &cinfo);
#endif

  /* Disable signal catcher. */
#ifdef NEED_SIGNAL_CATCHER
  enable_signal_catcher((j_common_ptr) NULL);
#endif

  return file_index;
}


/*
 * The main program.
 */

int
main (int argc, char **argv)
{
  int file_index;
    
  cOUT_Wild_Flag = 0;



  /* On Mac, fetch a command line. */
#ifdef USE_CCOMMAND
  argc = ccommand(&argv);
#endif

#ifdef MSDOS
  lsProgName = "djpeg";        /* DOS tends to be too verbose about argv[0] */
#else
  lsProgName = argv[0];
  if (lsProgName == NULL || lsProgName[0] == 0)
    lsProgName = "djpeg";        /* in case C library doesn't provide it */
#endif

  /* The default maxmem must be computed only once at program startup,
   * since releasing memory with free() won't give it back to the OS.
   */
#ifdef FREE_MEM_ESTIMATE
  default_maxmem = FREE_MEM_ESTIMATE;
#else
  default_maxmem = 0;
#endif

  /* Scan command line, parse switches and locate input file names */

  if (argc < 2)
    usage(0);            /* nothing on the command line?? */

  file_index = 0;

  while (file_index < argc-1)
    file_index = process_one_file(argc, argv, file_index);

  /* All done. */
  exit(EXIT_SUCCESS);
  return 0;            /* suppress no-return-value warnings */
}
