
// jpegtran.c

// Copyright (C) 1995-1997, Thomas G. Lane.
// This file is part of the Independent JPEG Group's software.
// For conditions of distribution and use, see the accompanying README file.

// This file contains a command-line user interface for JPEG transcoding.
// It is very similar to cjpeg.c, but provides lossless transcoding between
// different JPEG file formats.  It also provides some lossless and sort-of-
// lossless transformations of JPEG data.


#include <errno.h> 
#include <io.h>

  

#undef TWO_FILE_COMMANDLINE

#include "cdjpeg.h"      // Common decls for cjpeg/djpeg applications  
#include "transupp.h"      // Support routines for jpegtran  
#include "jversion.h"      // for version message  

#ifdef USE_CCOMMAND      // command-line reader for Macintosh  
#ifdef __MWERKS__
#include <SIOUX.h>              // Metrowerks needs this  
#include <console.h>      // ... and this  
#endif
#ifdef THINK_C
#include <console.h>      // Think declares it here  
#endif
#endif

#ifndef PATH_MAX         // ANSI maximum-pathname-length constant  
#define PATH_MAX 256
#endif




/*
 * Automatic determination of available memory.
 */

static long default_maxmem;         /* saves value determined at startup, or 0 */

#ifndef FREE_MEM_ESTIMATE         /* may be defined from command line */

#ifdef MSDOS                           /* For MS-DOS (unless flat-memory model) */

#include <dos.h>                  /* for access to intdos() call */

LOCAL(long)
unused_dos_memory (void)
/* Obtain total amount of unallocated DOS memory */
{
  union REGS regs;
  long nparas;

  regs.h.ah = 0x48;                  /* DOS function Allocate Memory Block */
  regs.x.bx = 0xFFFF;                  /* Ask for more memory than DOS can have */
  (void) intdos(&regs, &regs);
  /* DOS will fail and return # of paragraphs actually available in BX. */
  nparas = (unsigned int) regs.x.bx;
  /* Times 16 to convert to bytes. */
  return nparas << 4;
}

// The default memory setting is 95% of the available space. 
#define FREE_MEM_ESTIMATE  ((unused_dos_memory() * 95L) / 100L)

#endif /* MSDOS */

#ifdef ATARI                           /* For Atari ST/STE/TT, Pure C or Turbo C */

#include <ext.h>

/* The default memory setting is 90% of the available space. */
#define FREE_MEM_ESTIMATE  (((long) coreleft() * 90L) / 100L)

#endif /* ATARI */

/* Add memory-estimation procedures for other operating systems here,
 * with appropriate #ifdef's around them.
 */

#endif /* !FREE_MEM_ESTIMATE */


int iAnyKeyAlways=0;

void PressAnyKey() 
{
  char cT;
  fprintf(stderr, "--- press any key ---");
  getche(cT);
  fprintf(stderr, "\n");
}


Exit_Key(int P_RC)
{
  if (iAnyKeyAlways)
      PressAnyKey();
  exit(P_RC);
}

//
// Argument-parsing code.
// The switch parser is designed to be useful with DOS-style command line
// syntax, ie, intermixed switches and file names, where only the switches
// to the left of a given file name affect processing of that file.
  


static char *lsProgName;   // program name for error messages  

static char *lsOUT_FileName;   // for -outfile switch  
static char *lsOUT_NameBegin, *lsOUT_Tail, *lsOUT_Tail_Infer;
int  iDSN_NameLen = 0;
char *DSN_Spreader = &" ";
char cOUT_Wild_Flag;

char lsWORK_FileName[PATH_MAX];
int  iErrorFlag;


static JCOPY_OPTION copyoption;   // -copy switch  
static jpeg_transform_info transformoption; // image transformation options  


LOCAL(void) usage(char cP1)
// complain about bad command line  
{

 if (cP1 < 2)
 {
  fprintf(stderr, "usage: %s [switches] ", lsProgName);
//#ifdef TWO_FILE_COMMANDLINE
//  fprintf(stderr, "inputfile outputfile\n");
//#else
  fprintf(stderr, "inputfiles   (may contain wild cards)\n");
//#endif
 }

 if (cP1 == 0)
 {
  fprintf(stderr, "Switches (Abbreviations in capitals):\n");
 }
 if (cP1 == 0 || cP1 == 'o')
 {
  fprintf(stderr, "  -tail [string|OFF]  Add trailing string to name for output. [OFF=disable]\n");
  fprintf(stderr, "                      eg  _tran.jpg     [default infers from other parms]\n");
  fprintf(stderr, "  -Outfile name  Target (can be foldername ending in slash)\n");
 }

 //#ifdef ENTROPY_OPT_SUPPORTED
//  fprintf(stderr, "  -optimize      Optimize Huffman table (smaller file, but slow compression)\n");
//#endif

 if (cP1 == 0)
 {
#ifdef C_PROGRESSIVE_SUPPORTED
  fprintf(stderr, "  -Progressive   Progressive format - good for Net\n");
  fprintf(stderr, "  -Wait          Wait for anykey before exiting\n");
#endif
 }
 if (cP1 == 0 || cP1 == 'c')
 {
  fprintf(stderr, "  -Copy [None|Comments|All]   Which extra markers to copy from source\n");
 }


#if TRANSFORMS_SUPPORTED
 if (cP1 == 'm')
 {
  fprintf(stderr, "Switches for modifying the image:\n");
 }
 if (cP1 == 0 || cP1 == 'm')
 {
  fprintf(stderr, "  -GRayscale     Reduce to grayscale (omit color data)\n");
  fprintf(stderr, "  -Flip [Horizontal|Vertical]  Mirror image (left-right or top-bottom)\n");
  fprintf(stderr, "  -ROtate [90|180|270]         Rotate image (degrees clockwise)\n");
  fprintf(stderr, "  -m270          FlipHoriz + Rotate 270 transposition\n");
  fprintf(stderr, "  -m90e          FlipHoriz + Rotate 90  transversal\n");
  fprintf(stderr, "  -trim          Drop non-transformable edge blocks\n");
 }
#endif // TRANSFORMS_SUPPORTED  
 if (cP1 == 0 || cP1 == 'a')
 {
  fprintf(stderr, "Switches for advanced users:\n");
  fprintf(stderr, "  -Restart N     Set restart interval in rows, or in blocks with B\n");
  fprintf(stderr, "  -MAXmemory N   Maximum memory to use (in kbytes)\n");
//  fprintf(stderr, "  -verbose  or  -debug   Emit debug output\n");
 }

 if (cP1 == 0 || cP1 == 'w')
 {
  //fprintf(stderr, "Switches for wizards:\n");
//#ifdef C_ARITH_CODING_SUPPORTED
//  fprintf(stderr, "  -arithmetic    Use arithmetic coding\n");
//#endif
#ifdef C_MULTISCAN_FILES_SUPPORTED
  fprintf(stderr, "  -Scans file    Use multi-scan analysis parms from script file\n");
#endif
 }

  PressAnyKey();
  Exit_Key(EXIT_FAILURE);
}




LOCAL(void)
select_transform (JXFORM_CODE transform)
// Silly little routine to detect multiple transform options,
// which we can't handle.
  
{
#if TRANSFORMS_SUPPORTED
  if (transformoption.transform == JXFORM_NONE ||
      transformoption.transform == transform) {
    transformoption.transform = transform;
  } else {
    fprintf(stderr, "%s: can only do one image transformation at a time\n",
                         lsProgName);
    usage('m');
  }
#else
  fprintf(stderr, "%s: sorry, image transformation was not compiled\n",
     lsProgName);
  Exit_Key(EXIT_FAILURE);
#endif
}






void FileErrorMsg()
{
  char *lsErrTxt;

  /*
  if (errno == ENOPATH)
      lsErrTxt = &"Path Not Found";
  else
  if (errno == ENOFILE)
      lsErrTxt = &"File Not Found";
  else
  */
  if (errno == ENOENT)
      lsErrTxt = &"Path or File Not Found";
  else
  if (errno == 13)
      lsErrTxt = &"File currently LOCKED by another task";
  else
  if (errno == EACCES)
      lsErrTxt = &"Security Access Denied";
  else
  if (errno == 1)
      lsErrTxt = &"BAD DISK SECTOR - Permanent Error";
  else
  if (errno == EBADF)
      lsErrTxt = &"Bad File Number";
  else
      lsErrTxt = &" ";

  fprintf(stderr, "   %s\n", lsErrTxt);
  PressAnyKey();
}





/// Check for overwrite of an existing file; clear it with user

#ifndef NO_OVERWRITE_CHECK
int iOverWrite_Chk = 1;

LOCAL(boolean) is_overwrite_ok (char *outfname)
{
  FILE * ofile;
  int ch;

  ofile = fopen(outfname, READ_BINARY);
  if (ofile == NULL)
  {
     return TRUE;                  // not present  
  }

  fclose(ofile);                  // oops, it is present  

  for (;;) 
  {
    fprintf(stderr, "\rOutput File: %s%s already exists, overwrite it? [y/n/a/e] ",
             outfname, DSN_Spreader);
    fflush(stderr);

    ch = getche(stdin);
    fprintf(stderr, "\n");

    /*
    if (ch != '\n')                // flush rest of line 
      while (getc(stdin) != '\n')
         // nothing;
    */

    switch (ch) 
    {
    case 'A':
    case 'a':
      iOverWrite_Chk = 0;
    case 'Y':
    case 'y':
      return TRUE;

    // case '\n':
    case 'N':
    case 'n':
      return FALSE;

    // abort
    case 'E':
    case 'e':
    case 3:  // CTRL-C
        Exit_Key(EXIT_FAILURE);
    
    default:
      fprintf(stderr,"\nKeycode=%d\n", ch);
    // otherwise, ask again  
    }

  }
}

#endif



char *lsDSN_NameBegin, *lsDSN_Dot, cDSN_LastChar;
void DSN_Analyse(char *lsParmFileName)
{
  char cCurr, *lcCurr, cLast;

  lcCurr          = lsParmFileName;
  lsDSN_NameBegin = lcCurr;
  lsDSN_Dot       = 0;
  cLast           = 0;

  for (;;)
  {
    cCurr = *lcCurr;
    if (cCurr == 0)
        break;

    if (cCurr == '.')
        lsDSN_Dot = lcCurr;
    else
    if (cCurr == ':' || cCurr == '\\')
        lsDSN_NameBegin = lcCurr+1;
 
    cLast = cCurr;
    lcCurr++;        
  }

  // If no dot found, point to end of string marker
  if (lsDSN_Dot == 0)
      lsDSN_Dot = lcCurr;

  iDSN_NameLen = lcCurr - lsParmFileName;
  if (iDSN_NameLen > 25)
       DSN_Spreader = &"\n    ";

  cDSN_LastChar = cLast;

  return;
}






LOCAL(int)
parse_switches (j_compress_ptr cinfo, int argc, char **argv,
      int last_file_arg_seen, boolean for_real)
// Parse optional switches.
// Returns argv[] index of first file-name argument (== argc if none).
// Any file names with indexes <= last_file_arg_seen are ignored;
// they have presumably been processed in a previous iteration.
// (Pass 0 for last_file_arg_seen on the first or only iteration.)
// for_real is FALSE on the first (dummy) pass; we may skip any expensive
// processing.
  
{
  int argn;
  char *arg;
  boolean simple_progressive;
  char *scansarg = NULL;   // saves -scans parm if any  

  // Set up default JPEG parameters.  
  simple_progressive = FALSE;


  lsOUT_FileName = NULL;
  lsOUT_Tail     = NULL;  
  lsOUT_Tail_Infer  = NULL;
  cinfo->optimize_coding = TRUE; // OPTIMIZE IS ALWAYS A GOOD DEFAULT

  copyoption = JCOPYOPT_DEFAULT;
  transformoption.transform = JXFORM_NONE;
  transformoption.trim = FALSE;
  transformoption.force_grayscale = FALSE;
  cinfo->err->trace_level = 0;

  // Scan command line options, adjust parameters  

  for (argn = 1; argn < argc; argn++) 
  {
    arg = argv[argn];
    if (*arg != '-') 
    {
      // Not a switch, must be an input file name argument  
      if (argn <= last_file_arg_seen) 
      {
          if (!cOUT_Wild_Flag)
              lsOUT_FileName = NULL;   // -outfile applies to just one input file 
           continue;      // ignore this name if previously processed  
      }
      break;         // else done parsing switches  
    }
    arg++;         // advance past switch marker character  

    if (keymatch(arg, "arithmetic", 1)) 
    {
      // Use arithmetic coding.  
#ifdef C_ARITH_CODING_SUPPORTED
      cinfo->arith_code = TRUE;
          if (lsOUT_Tail_Infer == NULL)
              lsOUT_Tail_Infer  = &"_ari.jpg";
#else
      fprintf(stderr, "%s: sorry, arithmetic coding not supported\n",
         lsProgName);
      Exit_Key(EXIT_FAILURE);
#endif

    } 
    else 
    if (keymatch(arg, "copy", 1)) 
    {
      // Select which extra markers to copy.  
      if (++argn >= argc)   // advance to next argument  
          usage('c');
      if (keymatch(argv[argn], "none", 1)) 
      {
         copyoption = JCOPYOPT_NONE;
          if (lsOUT_Tail_Infer == NULL)
              lsOUT_Tail_Infer  = &"_c0.jpg";
      } 
      else 
      if (keymatch(argv[argn], "comments", 1)) 
      {
          copyoption = JCOPYOPT_COMMENTS;
          if (lsOUT_Tail_Infer == NULL)
              lsOUT_Tail_Infer  = &"_cc.jpg";
      } 
      else 
      if (keymatch(argv[argn], "all", 1)) 
      {
          copyoption = JCOPYOPT_ALL;
      } 
      else
          usage('c');

    } 
    else 
    if (keymatch(arg, "wait", 1)) 
    {
       iAnyKeyAlways = 1;
    } 
    else 
    if (keymatch(arg, "debug", 1)  || keymatch(arg, "dbg", 1) 
    ||  keymatch(arg, "verbose", 1)) 
    {
      // Enable debug printouts.  
      // On first -d, print version identification  
      static boolean printed_version = FALSE;

      if (! printed_version) 
      {
          fprintf(stderr, "Based on Independent JPEG Group's JPEGTRAN, version %s\n%s\n",
                  JVERSION, JCOPYRIGHT);
          printed_version = TRUE;
      }
      cinfo->err->trace_level++;

    } 
    else 
    if (keymatch(arg, "flip", 1)) 
    {
      // Mirror left-right or top-bottom.  
      if (++argn >= argc)   // advance to next argument  
          usage('m');
      if (keymatch(argv[argn], "horizontal", 1))
      {
          select_transform(JXFORM_FLIP_H);
          if (lsOUT_Tail_Infer == NULL)
              lsOUT_Tail_Infer  = &"_FHor.jpg";
      }
      else 
      if (keymatch(argv[argn], "vertical", 1))
      {
          select_transform(JXFORM_FLIP_V);
          if (lsOUT_Tail_Infer == NULL)
              lsOUT_Tail_Infer  = &"_FVert.jpg";
      }
      else
          usage('m');

    } 
    else 
    if (keymatch(arg, "grayscale", 2) || keymatch(arg, "greyscale",2)) 
    {
      // Force to grayscale.  
#if TRANSFORMS_SUPPORTED
      transformoption.force_grayscale = TRUE;
          if (lsOUT_Tail_Infer == NULL)
              lsOUT_Tail_Infer  = &"_gray.jpg";
#else
      select_transform(JXFORM_NONE);   // force an error  
#endif

    } 
    else 
    if (keymatch(arg, "maxmemory", 3)) 
    {
      // Maximum memory in Kb (or Mb with 'm').  
      long lval;
      char ch = 'x';

      if (++argn >= argc)   // advance to next argument  
          usage('a');
      if (sscanf(argv[argn], "%ld%c", &lval, &ch) < 1)
          usage('a');
      if (ch == 'a' || ch == 'M')
          lval *= 1000L;
      cinfo->mem->max_memory_to_use = lval * 1000L;

    } 
    else 
    if (keymatch(arg, "optimize", 3) || keymatch(arg, "optimise", 3)) 
    {
      // Enable entropy parm optimization.  
#ifdef ENTROPY_OPT_SUPPORTED
      cinfo->optimize_coding = TRUE;
#else
      fprintf(stderr, "%s: sorry, entropy optimization was not compiled\n",
         lsProgName);
      Exit_Key(EXIT_FAILURE);
#endif

    } 
    else 
    if (keymatch(arg, "nooptimize", 5) || keymatch(arg, "nooptimise", 5)) 
    {
      // Disable entropy parm optimization.  
      cinfo->optimize_coding = FALSE;
      if (lsOUT_Tail_Infer == NULL)
          lsOUT_Tail_Infer  = &"_noopt.jpg";

    } 
    else 
    if (keymatch(arg, "outfile", 1)) 
    {
      // Set output file name.  
      if (++argn >= argc)   // advance to next argument  
         usage('o');
      lsOUT_FileName = argv[argn];   // save it away for later use  

    } 
    else 
    if (keymatch(arg, "trail", 5)
    ||  keymatch(arg, "tail", 4)) 
    {
      // Set string for adding to output file name.  
      if (++argn >= argc)   // advance to next argument  
           usage('o');
      lsOUT_Tail = argv[argn];   // save it away for later use  

    } 
    else 
    if (keymatch(arg, "progressive", 1)) 
    {
      // Select simple progressive mode.  
#ifdef C_PROGRESSIVE_SUPPORTED
      simple_progressive = TRUE;
      if (lsOUT_Tail_Infer == NULL)
          lsOUT_Tail_Infer  = &"_P.jpg";

      // We must postpone execution until num_components is known.  
#else
      fprintf(stderr, "%s: sorry, progressive output was not compiled\n",
                             lsProgName);
      Exit_Key(EXIT_FAILURE);
#endif

    } 
    else 
    if (keymatch(arg, "restart", 1)) 
    {
      // Restart interval in MCU rows (or in MCUs with 'b').  
      long lval;
      char ch = 'x';
      if (lsOUT_Tail_Infer == NULL)
          lsOUT_Tail_Infer  = &"_rs.jpg";


      if (++argn >= argc)   // advance to next argument  
          usage('a');
      if (sscanf(argv[argn], "%ld%c", &lval, &ch) < 1)
          usage('a');
      if (lval < 0 || lval > 65535L)
          usage('a');
      if (ch == 'b' || ch == 'B') 
      {
          cinfo->restart_interval = (unsigned int) lval;
          cinfo->restart_in_rows = 0; // else prior '-restart n' overrides me  
      } 
      else 
      {
          cinfo->restart_in_rows = (int) lval;
   // restart_interval will be computed during startup  
      }

    } 
    else 
    if (keymatch(arg, "rotate", 2)) 
    {
      // Rotate 90, 180, or 270 degrees (measured clockwise).  
      if (++argn >= argc)   // advance to next argument  
          usage('m');
      if (keymatch(argv[argn], "90",  2)
      ||  keymatch(argv[argn], "090", 3))
      {
          select_transform(JXFORM_ROT_90);
          if (lsOUT_Tail_Infer == NULL)
              lsOUT_Tail_Infer  = &"_r090.jpg";
      }
      else 
      if (keymatch(argv[argn], "180", 3))
      {
          select_transform(JXFORM_ROT_180);
          if (lsOUT_Tail_Infer == NULL)
              lsOUT_Tail_Infer  = &"_r180.jpg";
      }
      else 
      if (keymatch(argv[argn], "270", 3))
      {
          select_transform(JXFORM_ROT_270);
          if (lsOUT_Tail_Infer == NULL)
              lsOUT_Tail_Infer  = &"_r270.jpg";
      }
      else
          usage('m');

    } 
    else 
    if (keymatch(arg, "Scans", 1)) 
    {
      // Set scan script.  
#ifdef C_MULTISCAN_FILES_SUPPORTED
      if (++argn >= argc)   // advance to next argument  
          usage('w');
      scansarg = argv[argn];
      if (lsOUT_Tail_Infer == NULL)
          lsOUT_Tail_Infer  = &"_sc.jpg";
      // We must postpone reading the file in case -progressive appears.  
#else
      fprintf(stderr, "%s: sorry, multi-scan output was not compiled\n",
                       lsProgName);
      Exit_Key(EXIT_FAILURE);
#endif

    } 
    else 
    if (keymatch(arg, "transpose", 1)
    ||  keymatch(arg, "m270", 4)) 
    {
      // Transpose (across UL-to-LR axis).  
      select_transform(JXFORM_TRANSPOSE);
      if (lsOUT_Tail_Infer == NULL)
          lsOUT_Tail_Infer  = &"_m270.jpg";

    } 
    else 
    if (keymatch(arg, "transverse", 6)
    ||  keymatch(arg, "m90",  3)
    ||  keymatch(arg, "m090", 4)) 
    {
      // Transverse transpose (across UR-to-LL axis).  
      select_transform(JXFORM_TRANSVERSE);
      if (lsOUT_Tail_Infer == NULL)
          lsOUT_Tail_Infer  = &"_m90.jpg";

    } 
    else 
    if (keymatch(arg, "trim", 3)) 
    {
      // Trim off any partial edge MCUs that the transform can't handle.  
      transformoption.trim = TRUE;
          if (lsOUT_Tail_Infer == NULL)
              lsOUT_Tail_Infer  = &"_trim.jpg";

    } 
    else 
    {
      fprintf(stderr, "\n%s: UNKNOWN KEYWORD %s\n\n",
               lsProgName, arg);
      usage(0);         // bogus switch  
    }
  }

  // Post-switch-scanning cleanup  

  if (for_real) 
  {

#ifdef C_PROGRESSIVE_SUPPORTED
    if (simple_progressive)   // process -progressive; -scans can override  
        jpeg_simple_progression(cinfo);
#endif

#ifdef C_MULTISCAN_FILES_SUPPORTED
    if (scansarg != NULL)   // process -scans if it was present  
        if (! read_scan_script(cinfo, scansarg))
              usage('w');
#endif
  }

  return argn;         // return index of next arg (file name)  
}






//

int process_one_file(int argc, char **argv, int prev_file)
{
  struct jpeg_decompress_struct srcinfo;
  struct jpeg_compress_struct dstinfo;
  struct jpeg_error_mgr jsrcerr, jdsterr;
#ifdef PROGRESS_REPORT
  struct cdjpeg_progress_mgr progress;
#endif
  jvirt_barray_ptr * src_coef_arrays;
  jvirt_barray_ptr * dst_coef_arrays;

  int file_index, iTmp1;
  int iInSize, iOutSize;

  FILE *input_file;
  FILE *output_file;

  // Initialize the JPEG decompression object with default error handling.  
  srcinfo.err = jpeg_std_error(&jsrcerr);
  jpeg_create_decompress(&srcinfo);
  // Initialize the JPEG compression object with default error handling.  
  dstinfo.err = jpeg_std_error(&jdsterr);
  jpeg_create_compress(&dstinfo);

  // Now safe to enable signal catcher.
  // Note: we assume only the decompression object will have virtual arrays.
    
#ifdef NEED_SIGNAL_CATCHER
  enable_signal_catcher((j_common_ptr) &srcinfo);
#endif

  // Scan command line to find file names.
  // It is convenient to use just one switch-parsing routine, but the switch
  // values read here are mostly ignored; we will rescan the switches after
  // opening the input file.  Also note that most of the switches affect the
  // destination JPEG object, so we parse into that and then copy over what
  // needs to affects the source too.

  if (argc == 0)
      usage(0);

  file_index = parse_switches(&dstinfo, argc, argv, prev_file, FALSE);

  jsrcerr.trace_level = jdsterr.trace_level;
  srcinfo.mem->max_memory_to_use = dstinfo.mem->max_memory_to_use;

  if (file_index >= argc
  ||  *argv[file_index] == 0) 
  {
    if (argc <= 1)
        iTmp1 = 0;
    else
    {
        fprintf(stderr, "%s: specify at least one input file\n", lsProgName);
        iTmp1 = 1;
    }
    usage(iTmp1);
  }
 

  // We may need to generate the output file or path or extension 
        
  if (lsOUT_Tail_Infer == NULL)
      lsOUT_Tail_Infer = &"_opt.jpg";


  // If user didn't supply -outfile switch, generate output file name.  
  if (lsOUT_FileName == NULL) 
  {
    // Make lsOUT_FileName be argv[file_index] with _extra.jpg substituted at end

    strcpy(lsWORK_FileName, argv[file_index]);

    // set output type to jpg
    DSN_Analyse(lsWORK_FileName);
    if (lsOUT_Tail == NULL
    ||  !stricmp(lsOUT_Tail,"default"))
    {
        lsOUT_Tail  = lsOUT_Tail_Infer;
    }
          
    if (stricmp(lsOUT_Tail,"OFF"))
        strcpy(lsDSN_Dot, lsOUT_Tail);

    lsOUT_FileName = lsWORK_FileName;
  }
  else
  {
    strcpy(lsWORK_FileName, lsOUT_FileName);
    DSN_Analyse(lsWORK_FileName);
    if (cDSN_LastChar == '\\')
    {
        // Outfile name is a path spec
        cOUT_Wild_Flag = 1;

        // Keep drive:path
        lsOUT_NameBegin = lsDSN_NameBegin;
        
        // check accessibility of output folder
        *lsOUT_NameBegin = 0;
        iErrorFlag = access(lsWORK_FileName, 2);
        if (iErrorFlag)
        {
            fprintf(stderr, "\nCannot access output folder. Err=%d\nOut= %s\n", 
                                            errno, lsWORK_FileName);
            FileErrorMsg();
            Exit_Key(EXIT_FAILURE);
        }

        // copy File Name.Ext from input 
        DSN_Analyse(argv[file_index]);
        strcpy (lsOUT_NameBegin, lsDSN_NameBegin);

        // set output type to jpg
        DSN_Analyse(lsWORK_FileName);
        if (lsOUT_Tail != NULL)
        {
            if (!stricmp(lsOUT_Tail,"default"))
                lsOUT_Tail  = lsOUT_Tail_Infer;

            if (!stricmp(lsOUT_Tail,"OFF"))
                strcpy(lsDSN_Dot, lsOUT_Tail);
        }

        lsOUT_FileName = lsWORK_FileName;
    }
    else
      cOUT_Wild_Flag = 0;
  }


  if (!stricmp(argv[file_index], lsOUT_FileName))
  {
      fprintf(stderr, "\rSAME INPUT AND OUTPUT FILE NAMES.\nFile or folder names or extension need to be different\n In= %s\nOut= %s\n", 
                                            argv[file_index], lsOUT_FileName);
      PressAnyKey();
      Exit_Key(EXIT_FAILURE);
  }

  fprintf(stderr, "\rTransforming %s%s => %s\n", 
                     argv[file_index], DSN_Spreader, 
                     lsOUT_FileName);


/* OLD CODE

#ifdef TWO_FILE_COMMANDLINE
  // Must have either -outfile switch or explicit output file name 
  if (lsOUT_FileName == NULL) 
  {
    if (file_index != argc-2) 
    {
      fprintf(stderr, "%s: must name one input and one output file\n",
         lsProgName);
              usage();
    }
    lsOUT_FileName = argv[file_index+1];
  } 
  else 
  {
    if (file_index != argc-1) 
    {
      fprintf(stderr, "%s: must name one input and one output file\n",
         lsProgName);
              usage();
    }
  }
#else
  // Unix style: expect zero or one file name 
  if (file_index < argc-1) 
  {
    fprintf(stderr, "%s: only one input file\n", lsProgName);
            usage();
  }
#endif // TWO_FILE_COMMANDLINE 

*/ 

  // Open the input file.  
  if (file_index < argc) 
  {
    if ((input_file = fopen(argv[file_index], READ_BINARY)) == NULL) 
    {
      fprintf(stderr, "%s: can't open %s\n", lsProgName, argv[file_index]);
      FileErrorMsg();
      Exit_Key(EXIT_FAILURE);
    }
  } 
  else 
  {
    // default input file is stdin  
    input_file = read_stdin();
  }

#ifndef NO_OVERWRITE_CHECK
  if (iOverWrite_Chk)
  {
    if(! is_overwrite_ok(lsOUT_FileName))
       goto fail;
  }
#endif

  // Open the output file.  
  if (lsOUT_FileName != NULL) 
  {
    if ((output_file = fopen(lsOUT_FileName, WRITE_BINARY)) == NULL) 
    {
      fprintf(stderr, "%s: can't open %s\n", lsProgName, lsOUT_FileName);
      FileErrorMsg();
      Exit_Key(EXIT_FAILURE);
    }
  } 
  else 
  {
    // default output file is stdout  
    output_file = write_stdout();
  }

#ifdef PROGRESS_REPORT
  start_progress_monitor((j_common_ptr) &dstinfo, &progress);
#endif

  // Specify data source for decompression  
  jpeg_stdio_src(&srcinfo, input_file);

  // Enable saving of extra markers that we want to copy  
  jcopy_markers_setup(&srcinfo, copyoption);

  // Read file header  
  (void) jpeg_read_header(&srcinfo, TRUE);

  // Any space needed by a transform option must be requested before
  // jpeg_read_coefficients so that memory allocation will be done right.
    
#if TRANSFORMS_SUPPORTED
  jtransform_request_workspace(&srcinfo, &transformoption);
#endif

  // Read source file as DCT coefficients  
  src_coef_arrays = jpeg_read_coefficients(&srcinfo);

  // Initialize destination compression parameters from source values  
  jpeg_copy_critical_parameters(&srcinfo, &dstinfo);

  // Adjust destination parameters if required by transform options;
  // also find out which set of coefficient arrays will hold the output.
    
#if TRANSFORMS_SUPPORTED
  dst_coef_arrays = jtransform_adjust_parameters(&srcinfo, &dstinfo,
                   src_coef_arrays,
                   &transformoption);
#else
  dst_coef_arrays = src_coef_arrays;
#endif

  // Adjust default compression parameters by re-parsing the options  
  file_index = parse_switches(&dstinfo, argc, argv, 0, TRUE);

  // Specify data destination for compression  
  jpeg_stdio_dest(&dstinfo, output_file);

  // Start compressor (note no image data is actually written here)  
  jpeg_write_coefficients(&dstinfo, dst_coef_arrays);

  // Copy to the output file any extra markers that we want to preserve  
  jcopy_markers_execute(&srcinfo, &dstinfo, copyoption);

  // Execute image transformation, if any  
#if TRANSFORMS_SUPPORTED
  jtransform_execute_transformation(&srcinfo, &dstinfo,
                src_coef_arrays,
                &transformoption);
#endif

  // Finish compression and release memory  
  jpeg_finish_compress(&dstinfo);

  if (input_file != stdin)
      iInSize = (ftell(input_file) +1023) / 1024;
  if (output_file != stdout)
      iOutSize = (ftell(output_file) +1023) / 1024;

  fprintf(stderr, "\r             %ik -> %ik\n",
                      iInSize, iOutSize);


fail:
  jpeg_destroy_compress(&dstinfo);
  (void) jpeg_finish_decompress(&srcinfo);
  jpeg_destroy_decompress(&srcinfo);

  // Close files, if we opened them  
  if (input_file != stdin)
    fclose(input_file);
  if (output_file != stdout)
    fclose(output_file);

#ifdef PROGRESS_REPORT
  end_progress_monitor((j_common_ptr) &dstinfo);
#endif

  // All done.  
  Exit_Key(jsrcerr.num_warnings + jdsterr.num_warnings ?EXIT_WARNING:EXIT_SUCCESS);
  return 0;         // suppress no-return-value warnings  
}



// higher level mainline added for multiple input file name processing

int main (int argc, char **argv)
{
int file_index; 

  // On Mac, fetch a command line.  
#ifdef USE_CCOMMAND
  argc = ccommand(&argv);
#endif

  lsProgName = argv[0];
  if (lsProgName == NULL || lsProgName[0] == 0)
    lsProgName = "JpegTRJ";   // in case C library doesn't provide it  

  // The default maxmem must be computed only once at program startup,
  // since releasing memory with free() won't give it back to the OS.
  
#ifdef FREE_MEM_ESTIMATE
  default_maxmem = FREE_MEM_ESTIMATE;
#else
  default_maxmem = 0;
#endif


  /* Scan command line, parse switches and locate input file names */

  if (argc < 2)
    usage(0);                           /* nothing on the command line?? */

  file_index = 0;

  while (file_index < argc-1)
    file_index = process_one_file(argc, argv, file_index);

  /* All done. */
  Exit_Key(EXIT_SUCCESS);
  return 0;                  
}
